package token_test

import (
	"testing"

	"git.akyoto.dev/cli/q/src/build/token"
	"git.akyoto.dev/go/assert"
)

func TestTokenEnd(t *testing.T) {
	hello := token.Token{
		Kind:     token.Identifier,
		Bytes:    []byte("hello"),
		Position: 0,
	}

	assert.Equal(t, hello.End(), 5)
}

func TestTokenReset(t *testing.T) {
	hello := token.Token{
		Kind:     token.Identifier,
		Bytes:    []byte("hello"),
		Position: 1,
	}

	hello.Reset()
	assert.Nil(t, hello.Bytes)
	assert.Equal(t, hello.Position, 0)
	assert.Equal(t, hello.Kind, token.Invalid)
}

func TestTokenString(t *testing.T) {
	hello := token.Token{
		Kind:     token.Identifier,
		Bytes:    []byte("hello"),
		Position: 0,
	}

	assert.Equal(t, hello.String(), "Identifier hello")
}

func TestTokenText(t *testing.T) {
	hello := token.Token{Kind: token.Identifier, Bytes: []byte("hello"), Position: 0}
	comma := token.Token{Kind: token.Separator, Bytes: []byte(","), Position: 5}
	world := token.Token{Kind: token.Identifier, Bytes: []byte("world"), Position: 7}

	assert.Equal(t, hello.Text(), "hello")
	assert.Equal(t, world.Text(), "world")

	list := token.List{hello, comma, world}
	assert.Equal(t, list.String(), "hello, world")
}
